@@ -62,6 +62,8 @@
      m_set_lisp_loc
      m_tcl_set_text_variable
      m_tcl_unlink_text_variable
+     m_lisp_eval
+     m_lisp_eval_wait_response
      ))
 
 (defconstant *magic1* #\)
@@ -571,9 +573,12 @@
 		  pathname option ))
        (pp option no_quote)
        (setq l (cdr l))
+       ;(print (car l))
        (cond ((and (keywordp (car l))
 		   (not (eq option :configure))
 		   (not (eq option :config))
+		   (not (eq option :itemconfig))
+		   (not (eq option :cget))
 		   (not (eq option :postscript))
 			)
 	      (pp (car l) no_quote)
@@ -795,6 +800,10 @@
 (def-control tk-dialog :print-name "tk_dialog")
 (def-control get_tag_range)
 
+(def-control lsearch)
+(def-control lindex)
+
+
 (defun tk-wait-til-exists (win)
   (tk-do (tk-conc "if ([winfo exists " win " ]) { } else {tkwait visibility " win "}")))
 
@@ -948,7 +957,9 @@
 		       (subseq vec 0 (length *header*))
 	       )
        )
-      ((m_create_command m_call)
+      ((m_create_command m_call
+         m_lisp_eval
+	m_lisp_eval_wait_response)
        (let ((islot (get-number-string vec #.(+ 0 (length *header*)) 3)))
 	 (format t "islot=~a(callback_~a), arglist=~s" islot  islot
 		 (subseq vec #.(+ 3 (length *header*))))))
@@ -972,6 +983,7 @@
 		 (setq var (aref *text-variable-locations* islot))
 		 (get var 'linked-variable-type)
 		 (subseq vec #.(+ 3 (length *header*))))))
+      
       (otherwise (error "unknown message type ~a [~s]" mtype vec )))))
 
 (defun clear-tk-connection ()
@@ -1399,12 +1411,12 @@
 	       ((probe-file (tk-conc si::*lib-directory* "gcl-tk/gcltksrv")))
 	       (t (error "Must setenv GCL_TK_SERVER ")))))
   (let ((pid (if host  -1 (si::getpid)))
-	(tk-socket  (si::open-named-socket 0 nil))
+	(tk-socket  (si::open-named-socket 0))
 	)
     (cond ((not host) (setq hostid loopback))
 	  (host (setq hostid (si::hostname-to-hostid (si::gethostname)))))
     (or hostid (error "Can't find my address"))
-    (setq tk-socket (si::open-named-socket 0 nil))
+    (setq tk-socket (si::open-named-socket 0))
     (if (pathnamep gcltksrv) (setq gcltksrv (namestring gcltksrv)))
     (let ((command 
 	   (tk-conc   gcltksrv " " hostid " "
@@ -1412,6 +1424,7 @@
 		        pid " " display " "
 			args
 			)))
+      (print command)
       (cond ((not host) (system command))
 	    (can-rsh
 	      (system (tk-conc "rsh " host " "   command
