@@ -27,7 +27,7 @@ Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
 #include "num_include.h"
 
-static void
+void
 zero_divisor(void);
 
 object fixnum_add(int i, int j)
@@ -819,7 +819,6 @@ integer_divide1(object x, object y)
 object
 get_gcd(object x, object y)
 {
-	int	i, j, k;
 	object	q, r;
 
 	if (number_minusp(x))
@@ -829,21 +828,40 @@ get_gcd(object x, object y)
 
 L:
 	if (type_of(x) == t_fixnum && type_of(y) == t_fixnum) {
-		i = fix(x);
-		j = fix(y);
-LL:
-		if (i < j) {
-			k = i;
-			i = j;
-			j = k;
-		}
-		if (j == 0) {
-			return(make_fixnum(i));
-		}
-		k = i % j;
-		i = j;
-		j = k;
-		goto LL;
+	  /* LL: */
+	  /* 		if (i < j) { */
+	  /* 			k = i; */
+	  /* 			i = j; */
+	  /* 			j = k; */
+	  /* 		} */
+	  /* 		if (j == 0) { */
+	  /* 			return(make_fixnum(i)); */
+	  /* 		} */
+	  /* 		k = i % j; */
+	  /* 		i = j; */
+	  /* 		j = k; */
+	  /* 		goto LL; */
+	  
+	  register fixnum i, j, k, t;
+
+	  if (!(i = fix(x)))
+	    return y;
+	  if (!(j = fix(y)))
+	    return x;
+	  k=0;
+	  while(!(i&0x1) && !(j&0x1)) {
+	      k++;
+	      i>>=1;
+	      j>>=1;
+	  }
+	  t= i&0x1 ? -j : i>>1; 
+	  do {
+	    while(!(t&0x1)) t>>=1;
+	    if(t>0) i=t; else j=-t;
+	    t=i-j;
+	  } while (t);
+	  return make_fixnum(i<<k);
+
 	}
 
 	if (number_compare(x, y) < 0) {
@@ -861,8 +879,7 @@ LL:
 }
 
 /* (+          )   */
-void
-Lplus(void)
+LFD(Lplus)(void)
 {
         int i, j;
 	
@@ -878,8 +895,7 @@ Lplus(void)
 	vs_top = vs_base+1;
 }
 
-void
-Lminus(void)
+LFD(Lminus)(void)
 {
 	int i, j;
 
@@ -897,8 +913,7 @@ Lminus(void)
 	vs_top = vs_base+1;
 }
 
-void
-Ltimes(void)
+LFD(Ltimes)(void)
 {
 	int i, j;
 
@@ -914,8 +929,7 @@ Ltimes(void)
 	vs_top = vs_base+1;
 }
 
-void
-Ldivide(void)
+LFD(Ldivide)(void)
 {
 	int i, j;
 
@@ -933,8 +947,7 @@ Ldivide(void)
 	vs_top = vs_base+1;
 }
 
-void
-Lone_plus(void)
+LFD(Lone_plus)(void)
 {
 	
 	check_arg(1);
@@ -942,8 +955,7 @@ Lone_plus(void)
 	vs_base[0] = one_plus(vs_base[0]);
 }
 
-void
-Lone_minus(void)
+LFD(Lone_minus)(void)
 {
 	
 	check_arg(1);
@@ -951,8 +963,7 @@ Lone_minus(void)
 	vs_base[0] = one_minus(vs_base[0]);
 }
 
-void
-Lconjugate(void)
+LFD(Lconjugate)(void)
 {
 	object	c, i;
 
@@ -967,8 +978,7 @@ Lconjugate(void)
 	}
 }
 
-void
-Lgcd(void)
+LFD(Lgcd)(void)
 {
 	int i, narg;
 
@@ -989,8 +999,7 @@ Lgcd(void)
 	vs_top = vs_base+1;
 }
 
-void
-Llcm(void)
+LFD(Llcm)(void)
 {
 	object t, g;
 	int i, narg;
@@ -1006,27 +1015,32 @@ Llcm(void)
 		return;
 	}
 	for (i = 1;  i < narg;  i++) {
-		t = number_times(vs_base[0], vs_base[i]);
-		vs_push(t);
-		g = get_gcd(vs_base[0], vs_base[i]);
-		vs_push(g);
-		vs_base[0] = number_divide(t, g);
-		vs_popp;
-		vs_popp;
+	  if (number_zerop(vs_base[0]) ||
+	      number_zerop(vs_base[i]))
+	    vs_base[0]=small_fixnum(0);
+	  else {
+	    t = number_times(vs_base[0], vs_base[i]);
+	    vs_push(t);
+	    g = get_gcd(vs_base[0], vs_base[i]);
+	    vs_push(g);
+	    vs_base[0] = number_divide(t, g);
+	    vs_popp;
+	    vs_popp;
+	  }
 	}
 	if (number_minusp(vs_base[0]))
 		vs_base[0] = number_negate(vs_base[0]);
 	vs_top = vs_base+1;
 }
 
-static void
+void
 zero_divisor(void)
 {
 	FEerror("Zero divisor.", 0);
 }
 
 void
-init_num_arith(void)
+gcl_init_num_arith(void)
 {
 	make_function("+", Lplus);
 	make_function("-", Lminus);
