@@ -29,12 +29,13 @@ License for more details.
 
 /* #include "include.h" */
 #include "ext_sym.h"
+#include <string.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <sys/types.h>
 #include <sys/mman.h>
 #include <sys/stat.h>
-
+#include <string.h>
 
 #include <elf.h>
 
@@ -78,19 +79,20 @@ read_special_symbols(symfile)
 char *symfile;
 {FILE *symin;
  char *symbols;
- int i,jj;
+ int i
+ unsigned long jj;
  struct lsymbol_table tab;
  if (!(symin=fopen(symfile,"r")))
    {perror(symfile);exit(1);};
  if(!fread((char *)&tab,sizeof(tab),1,symin))
-   FEerror("No header",0,0);
+   FEerror("No header",0);
  symbols=malloc(tab.tot_leng);
  c_table.alloc_length=( (PTABLE_EXTRA+ tab.n_symbols));
  (c_table.ptable) = (TABL *) malloc(sizeof(struct node) * c_table.alloc_length);
  if (!(c_table.ptable)) {perror("could not allocate"); exit(1);};
  i=0; c_table.length=tab.n_symbols;
  while(i < tab.n_symbols)
-   { fread((char *)&jj,sizeof(int),1,symin);
+   { fread((char *)&jj,sizeof(jj),1,symin);
      (SYM_ADDRESS(c_table,i))=jj;
      SYM_STRING(c_table,i)=symbols;
  
@@ -203,7 +205,7 @@ fasload(faslfile)
      base = mmap (0, stat_buf.st_size,PROT_READ|PROT_WRITE, MAP_PRIVATE, file, 0); } else {
 
    base = malloc(stat_buf.st_size);
-   fread(base,1,stat_buf.st_size,fp);
+   SAFE_FREAD(base,1,stat_buf.st_size,fp);
    fseek(fp,0,SEEK_CUR);
      }
 
@@ -303,7 +305,7 @@ fasload(faslfile)
    
    relocate_symbols(symbol_table,nsyms,file_h->e_shnum,&init_address);
    if (init_address < 0)
-     { FEerror(0,"Init address not found ");
+     { FEerror("Init address not found ",0);
      }
 
    { 
@@ -317,7 +319,7 @@ fasload(faslfile)
 	   {
 	     int index_to_relocate = shp->sh_info;
 	     if (symtab_index != shp->sh_link)
-	       FEerror("unexpected symbol table used");
+	       FEerror("unexpected symbol table used",0);
 	     the_start = start_address + section[index_to_relocate].start;
 	   }
 /*  	 else if (shp->sh_type == SHT_REL */
@@ -330,9 +332,13 @@ fasload(faslfile)
 /*  	   } */
 	 
 	 else if ( (shp->sh_type == SHT_REL) || (shp->sh_type == SHT_RELA) )
-	   {  if (get_section_number(".rel.stab") == j)
+	   {  if (/* get_section_number(".rel.stab") == j || */
+		  /* Newer gcc uses these section name -- CMM 20040224*/
+		  !strncmp(section_names+SECTION_H(j).sh_name,".rel.debug",10) ||
+		  /* old in for backward compatibility */
+		  !strcmp(section_names+SECTION_H(j).sh_name,".rel.stab"))
 	         continue;
-	     FEerror("unknown rel type");
+	     FEerror("unknown rel type",0);
 	   }
 	 else
 	   continue;
@@ -520,7 +526,7 @@ Elf32_Word sh_type;
     else if (sh_type == SHT_REL)
       a = 0;
     else {
-      FEerror("relocate() error: unknown sh_type in ELF object");
+      FEerror("relocate() error: unknown sh_type in ELF object",0);
       a=0;
     }
     b = (unsigned int) the_start;
@@ -540,9 +546,9 @@ Elf32_Word sh_type;
 /*      do_bfd_reloc(ELF32_R_TYPE(reloc_info->r_info),s+a,(unsigned int *)where); */
 /*  #else */
     switch(ELF32_R_TYPE(reloc_info->r_info)){
-#if (defined(__svr4__) || defined(__linux__)) && defined(__i386__)
+#if (defined(__svr4__) || defined(__linux__) || defined(__FreeBSD__)) && defined(__i386__)
     case     R_386_NONE:
-      FEerror("Unsupported ELF type R_386_NONE");
+      FEerror("Unsupported ELF type R_386_NONE",0);
       break;
 
     case     R_386_32:
@@ -556,40 +562,40 @@ Elf32_Word sh_type;
       break;
 
     case     R_386_GOT32:
-      FEerror("Unsupported ELF type R_386_GOY32");
+      FEerror("Unsupported ELF type R_386_GOY32",0);
       break;
 
     case     R_386_PLT32:
-      FEerror("Unsupported ELF type R_386_PLT32");
+      FEerror("Unsupported ELF type R_386_PLT32",0);
       break;
 
     case     R_386_COPY:
-      FEerror("Unsupported ELF type R_386_COPY");
+      FEerror("Unsupported ELF type R_386_COPY",0);
       break;
 
     case     R_386_GLOB_DAT:
-      FEerror("Unsupported ELF type R_386_GLOB_DAT");
+      FEerror("Unsupported ELF type R_386_GLOB_DAT",0);
       break;
 
     case     R_386_JMP_SLOT:
-      FEerror("Unsupported ELF type R_386_JMP_SLOT");
+      FEerror("Unsupported ELF type R_386_JMP_SLOT",0);
       break;
 
     case     R_386_RELATIVE:
-      FEerror("Unsupported ELF type R_386_RELATIVE");
+      FEerror("Unsupported ELF type R_386_RELATIVE",0);
       break;
 
     case     R_386_GOTOFF:
-      FEerror("Unsupported ELF type R_386_GOTOFF");
+      FEerror("Unsupported ELF type R_386_GOTOFF",0);
       break;
 
     case     R_386_GOTPC:
-      FEerror("Unsupported ELF type R_386_GOTPC");
+      FEerror("Unsupported ELF type R_386_GOTPC",0);
       break;
 
 #ifdef R_386_NUM
     case     R_386_NUM:
-      FEerror("Unsupported ELF type R_386_NUM");
+      FEerror("Unsupported ELF type R_386_NUM",0);
       break;
 #endif
 
@@ -657,7 +663,7 @@ char *string;
 	  fflush(stdout);
 	}
     }
-  else{FEerror("symbol table not loaded",0,0);}
+  else{FEerror("symbol table not loaded",0);}
 }
 
 static void
